#include "tilemap.h"
#include "program.h"
#include <SDL2/SDL_image.h>
#include <stdio.h>

struct Tilemap tilemap_load(const char* tilemap_file, const char* tileset_file) {
    struct Tilemap self = {
        .dimensions = {1,1},
        .tiles = NULL
    };

    FILE* fs = fopen(tilemap_file, "r");
    int n = 0;
    do {
        n = fgetc(fs);
        if(n == ',') {
            self.dimensions.x++;
        }
    } while(n != '\n');
    do {
        n = fgetc(fs);
        if(n == '\n') {
            self.dimensions.y++;
        }
    } while(n != '\0');

    rewind(fs);

    self.tiles = malloc(self.dimensions.x * self.dimensions.y * sizeof(int));

    char csv_buffer[6];
    csv_buffer[5] = '\0';
    char* writer = csv_buffer;
    int* tile = self.tiles;

    do {
        n = fgetc(fs);
        if(isalnum(n)) {
            *writer = n;
            ++writer;
        } else if(n == ',' || n == '\0' || n == '\n') {
            *writer = '\0';
            writer = csv_buffer;
            *tile = atoi(csv_buffer);
            ++tile;
        }
    } while(n != '\0');

    fclose(fs);

    self.tileset = tileset_load(tileset_file);
    
    return self;
}

static inline
int is_path_char(int n) {
    return isalnum(n) || n == '/';
}

struct Tileset tileset_load(const char* filename) {
    char* texture_path = NULL;
    char* writer = NULL;
    char num_buffer[6];
    num_buffer[5] = '\0';
    int n = 0;
    struct Tileset self = {
        .texture = NULL,
        .tile_size = {0,0}
    };
    size_t len = 1;

    FILE* fs = fopen(filename, "r");

    do {
        n = fgetc(fs);
        if(is_path_char(n)) {
            ++len;
        }
    } while(n != ',');

    texture_path = malloc(len * sizeof(char));

    rewind(fs);
    writer = texture_path;

    do {
        n = fgetc(fs);
        if(is_path_char(n)) {
            *writer = n;
            ++writer;
        } else if(n == ',') {
            *writer = '\0';
        }
    } while(n != ',');

    self.texture = IMG_LoadTexture(g_program.render_context->renderer, texture_path);
    SDL_QueryTexture(self.texture, NULL, NULL, &self.texture_resolution.x, &self.texture_resolution.y);
    self.shear = floor((float)self.texture_resolution.x / self.tile_size.x);

    writer = num_buffer;

    do {
        n = fgetc(fs);
        if(isdigit(n) || n == '-') {
            *writer= n;
            ++writer;
        } else if(n == ',' || feof(fs) || n == '\n') {
            *writer = '\0';
            writer = num_buffer;
            if(self.tile_size.x == 0) {
                self.tile_size.x = atoi(num_buffer);
            } else {
                self.tile_size.y = atoi(num_buffer);
            }
        }
    } while(!feof(fs));

    fclose(fs);

    return self;
}

SDL_Rect tileset_index_to_rect(struct Tileset* self, size_t index) {
    SDL_Rect rect = {0,0, self->tile_size.x, self->tile_size.y};
    rect.x = (index % self->shear) * rect.w;
    rect.y = (index / self->shear) * rect.h;
    return rect;
}

void tilemap_render(struct Tilemap* self, struct RenderContext* render) {
    size_t num_tiles = self->dimensions.x * self->dimensions.y;
    SDL_Rect source_rect = {0, 0, self->tileset.tile_size.x, self->tileset.tile_size.y};
    SDL_Rect dest_rect = source_rect;
    for(int i = 0; i < num_tiles; ++i) {
        source_rect = tileset_index_to_rect(&self->tileset, self->tiles[i]);
        dest_rect.x = (i % self->dimensions.x) * dest_rect.w;
        dest_rect.y = (i / self->dimensions.y) * dest_rect.h;
        SDL_RenderCopy(render->renderer, self->tileset.texture, &source_rect, &dest_rect);
    }
}